# Standard library imports
from __future__ import annotations

# Third party imports
import opengeode as og
import geode_viewables as viewables
from opengeodeweb_microservice.database.data_types import GeodeMeshType

# Local application imports
from .geode_surface_mesh3d import GeodeSurfaceMesh3D


class GeodePolygonalSurface3D(GeodeSurfaceMesh3D):
    polygonal_surface: og.PolygonalSurface3D

    def __init__(self, polygonal_surface: og.PolygonalSurface3D | None = None) -> None:
        self.polygonal_surface = (
            polygonal_surface
            if polygonal_surface is not None
            else og.PolygonalSurface3D.create()
        )
        super().__init__(self.polygonal_surface)

    @classmethod
    def geode_object_type(cls) -> GeodeMeshType:
        return "PolygonalSurface3D"

    def native_extension(self) -> str:
        return self.polygonal_surface.native_extension()

    def builder(self) -> og.PolygonalSurfaceBuilder3D:
        return og.PolygonalSurfaceBuilder3D.create(self.polygonal_surface)

    @classmethod
    def load(cls, filename: str) -> GeodePolygonalSurface3D:
        return GeodePolygonalSurface3D(og.load_polygonal_surface3D(filename))

    @classmethod
    def additional_files(cls, filename: str) -> og.AdditionalFiles:
        return og.polygonal_surface_additional_files3D(filename)

    @classmethod
    def is_loadable(cls, filename: str) -> og.Percentage:
        return og.is_polygonal_surface_loadable3D(filename)

    @classmethod
    def input_extensions(cls) -> list[str]:
        return og.PolygonalSurfaceInputFactory3D.list_creators()

    @classmethod
    def output_extensions(cls) -> list[str]:
        return og.PolygonalSurfaceOutputFactory3D.list_creators()

    @classmethod
    def object_priority(cls, filename: str) -> int:
        return og.polygonal_surface_object_priority3D(filename)

    def is_saveable(self, filename: str) -> bool:
        return og.is_polygonal_surface_saveable3D(self.polygonal_surface, filename)

    def save(self, filename: str) -> list[str]:
        return og.save_polygonal_surface3D(self.polygonal_surface, filename)

    def save_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_viewable_polygonal_surface3D(
            self.polygonal_surface, filename_without_extension
        )

    def save_light_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_light_viewable_polygonal_surface3D(
            self.polygonal_surface, filename_without_extension
        )



# Standard library imports
from __future__ import annotations

# Third party imports
import opengeode as og
import opengeode_geosciences as og_geosciences
import opengeode_inspector as og_inspector
from opengeodeweb_microservice.database.data_types import ViewerElementsType

# Local application imports
from .geode_vertex_set import GeodeVertexSet


class GeodeSurfaceMesh3D(GeodeVertexSet):
    surface_mesh: og.SurfaceMesh3D

    def __init__(self, surface_mesh: og.SurfaceMesh3D | None = None) -> None:
        self.surface_mesh = (
            surface_mesh if surface_mesh is not None else og.SurfaceMesh3D.create()
        )
        super().__init__(self.surface_mesh)

    @classmethod
    def is_3D(cls) -> bool:
        return True

    @classmethod
    def is_viewable(cls) -> bool:
        return True

    @classmethod
    def viewer_elements_type(cls) -> ViewerElementsType:
        return "polygons"

    def builder(self) -> og.SurfaceMeshBuilder3D:
        return og.SurfaceMeshBuilder3D.create(self.surface_mesh)

    def inspect(self) -> og_inspector.SurfaceInspectionResult:
        return og_inspector.inspect_surface3D(self.surface_mesh)

    def assign_crs(
        self, crs_name: str, info: og_geosciences.GeographicCoordinateSystemInfo
    ) -> None:
        builder = self.builder()
        og_geosciences.assign_surface_mesh_geographic_coordinate_system_info3D(
            self.surface_mesh, builder, crs_name, info
        )

    def convert_crs(
        self, crs_name: str, info: og_geosciences.GeographicCoordinateSystemInfo
    ) -> None:
        builder = self.builder()
        og_geosciences.convert_surface_mesh_coordinate_reference_system3D(
            self.surface_mesh, builder, crs_name, info
        )

    def create_crs(
        self, crs_name: str, input: og.CoordinateSystem2D, output: og.CoordinateSystem2D
    ) -> None:
        builder = self.builder()
        og.create_surface_mesh_coordinate_system3D(
            self.surface_mesh, builder, crs_name, input, output
        )

    def polygon_attribute_manager(self) -> og.AttributeManager:
        return self.surface_mesh.polygon_attribute_manager()

    def texture_manager(self) -> og.TextureManager2D:
        return self.surface_mesh.texture_manager()



# Standard library imports
from __future__ import annotations

# Third party imports
import opengeode as og
import opengeode_geosciences as og_geosciences
import geode_viewables as viewables
from opengeodeweb_microservice.database.data_types import GeodeModelType

# Local application imports
from .geode_structural_model import GeodeStructuralModel


class GeodeImplicitStructuralModel(GeodeStructuralModel):
    implicit_structural_model: og_geosciences.ImplicitStructuralModel

    def __init__(
        self,
        implicit_structural_model: og_geosciences.ImplicitStructuralModel | None = None,
    ) -> None:
        self.implicit_structural_model = (
            implicit_structural_model
            if implicit_structural_model is not None
            else og_geosciences.ImplicitStructuralModel()
        )
        super().__init__(self.implicit_structural_model)

    @classmethod
    def geode_object_type(cls) -> GeodeModelType:
        return "ImplicitStructuralModel"

    def native_extension(self) -> str:
        return self.implicit_structural_model.native_extension()

    def builder(self) -> og_geosciences.ImplicitStructuralModelBuilder:
        return og_geosciences.ImplicitStructuralModelBuilder(
            self.implicit_structural_model
        )

    @classmethod
    def load(cls, filename: str) -> GeodeImplicitStructuralModel:
        return GeodeImplicitStructuralModel(
            og_geosciences.load_implicit_structural_model(filename)
        )

    @classmethod
    def additional_files(cls, filename: str) -> og.AdditionalFiles:
        return og_geosciences.implicit_structural_model_additional_files(filename)

    @classmethod
    def is_loadable(cls, filename: str) -> og.Percentage:
        return og_geosciences.is_implicit_structural_model_loadable(filename)

    @classmethod
    def input_extensions(cls) -> list[str]:
        return og_geosciences.ImplicitStructuralModelInputFactory.list_creators()

    @classmethod
    def output_extensions(cls) -> list[str]:
        return og_geosciences.ImplicitStructuralModelOutputFactory.list_creators()

    @classmethod
    def object_priority(cls, filename: str) -> int:
        return og_geosciences.implicit_structural_model_object_priority(filename)

    def is_saveable(self, filename: str) -> bool:
        return og_geosciences.is_implicit_structural_model_saveable(
            self.implicit_structural_model, filename
        )

    def save(self, filename: str) -> list[str]:
        return og_geosciences.save_implicit_structural_model(
            self.implicit_structural_model, filename
        )

    def save_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_viewable_implicit_structural_model(
            self.implicit_structural_model, filename_without_extension
        )

    def save_light_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_light_viewable_implicit_structural_model(
            self.implicit_structural_model, filename_without_extension
        )



# Standard library imports
from __future__ import annotations

# Third party imports
import opengeode as og
import geode_viewables as viewables
from opengeodeweb_microservice.database.data_types import GeodeMeshType

# Local application imports
from .geode_surface_mesh2d import GeodeSurfaceMesh2D


class GeodeTriangulatedSurface2D(GeodeSurfaceMesh2D):
    triangulated_surface: og.TriangulatedSurface2D

    def __init__(
        self, triangulated_surface: og.TriangulatedSurface2D | None = None
    ) -> None:
        self.triangulated_surface = (
            triangulated_surface
            if triangulated_surface is not None
            else og.TriangulatedSurface2D.create()
        )
        super().__init__(self.triangulated_surface)

    @classmethod
    def geode_object_type(cls) -> GeodeMeshType:
        return "TriangulatedSurface2D"

    def native_extension(self) -> str:
        return self.triangulated_surface.native_extension()

    def builder(self) -> og.TriangulatedSurfaceBuilder2D:
        return og.TriangulatedSurfaceBuilder2D.create(self.triangulated_surface)

    @classmethod
    def load(cls, filename: str) -> GeodeTriangulatedSurface2D:
        return GeodeTriangulatedSurface2D(og.load_triangulated_surface2D(filename))

    @classmethod
    def additional_files(cls, filename: str) -> og.AdditionalFiles:
        return og.triangulated_surface_additional_files2D(filename)

    @classmethod
    def is_loadable(cls, filename: str) -> og.Percentage:
        return og.is_triangulated_surface_loadable2D(filename)

    @classmethod
    def input_extensions(cls) -> list[str]:
        return og.TriangulatedSurfaceInputFactory2D.list_creators()

    @classmethod
    def output_extensions(cls) -> list[str]:
        return og.TriangulatedSurfaceOutputFactory2D.list_creators()

    @classmethod
    def object_priority(cls, filename: str) -> int:
        return og.triangulated_surface_object_priority2D(filename)

    def is_saveable(self, filename: str) -> bool:
        return og.is_triangulated_surface_saveable2D(
            self.triangulated_surface, filename
        )

    def save(self, filename: str) -> list[str]:
        return og.save_triangulated_surface2D(self.triangulated_surface, filename)

    def save_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_viewable_triangulated_surface2D(
            self.triangulated_surface, filename_without_extension
        )

    def save_light_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_light_viewable_triangulated_surface2D(
            self.triangulated_surface, filename_without_extension
        )



# Standard library imports
from __future__ import annotations

# Third party imports
import opengeode as og
import geode_viewables as viewables
from opengeodeweb_microservice.database.data_types import GeodeMeshType

# Local application imports
from .geode_solid_mesh3d import GeodeSolidMesh3D


class GeodeHybridSolid3D(GeodeSolidMesh3D):
    hybrid_solid: og.HybridSolid3D

    def __init__(self, hybrid_solid: og.HybridSolid3D | None = None) -> None:
        self.hybrid_solid = (
            hybrid_solid if hybrid_solid is not None else og.HybridSolid3D.create()
        )
        super().__init__(self.hybrid_solid)

    @classmethod
    def geode_object_type(cls) -> GeodeMeshType:
        return "HybridSolid3D"

    def native_extension(self) -> str:
        return self.hybrid_solid.native_extension()

    def builder(self) -> og.HybridSolidBuilder3D:
        return og.HybridSolidBuilder3D.create(self.hybrid_solid)

    @classmethod
    def load(cls, filename: str) -> GeodeHybridSolid3D:
        return GeodeHybridSolid3D(og.load_hybrid_solid3D(filename))

    @classmethod
    def additional_files(cls, filename: str) -> og.AdditionalFiles:
        return og.hybrid_solid_additional_files3D(filename)

    @classmethod
    def is_loadable(cls, filename: str) -> og.Percentage:
        return og.is_hybrid_solid_loadable3D(filename)

    @classmethod
    def input_extensions(cls) -> list[str]:
        return og.HybridSolidInputFactory3D.list_creators()

    @classmethod
    def output_extensions(cls) -> list[str]:
        return og.HybridSolidOutputFactory3D.list_creators()

    @classmethod
    def object_priority(cls, filename: str) -> int:
        return og.hybrid_solid_object_priority3D(filename)

    def is_saveable(self, filename: str) -> bool:
        return og.is_hybrid_solid_saveable3D(self.hybrid_solid, filename)

    def save(self, filename: str) -> list[str]:
        return og.save_hybrid_solid3D(self.hybrid_solid, filename)

    def save_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_viewable_hybrid_solid3D(
            self.hybrid_solid, filename_without_extension
        )

    def save_light_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_light_viewable_hybrid_solid3D(
            self.hybrid_solid, filename_without_extension
        )



# Standard library imports
from __future__ import annotations

# Third party imports
import opengeode as og
import geode_viewables as viewables
from opengeodeweb_microservice.database.data_types import GeodeMeshType

# Local application imports
from .geode_surface_mesh3d import GeodeSurfaceMesh3D


class GeodeTriangulatedSurface3D(GeodeSurfaceMesh3D):
    triangulated_surface: og.TriangulatedSurface3D

    def __init__(
        self, triangulated_surface: og.TriangulatedSurface3D | None = None
    ) -> None:
        self.triangulated_surface = (
            triangulated_surface
            if triangulated_surface is not None
            else og.TriangulatedSurface3D.create()
        )
        super().__init__(self.triangulated_surface)

    @classmethod
    def geode_object_type(cls) -> GeodeMeshType:
        return "TriangulatedSurface3D"

    def native_extension(self) -> str:
        return self.triangulated_surface.native_extension()

    def builder(self) -> og.TriangulatedSurfaceBuilder3D:
        return og.TriangulatedSurfaceBuilder3D.create(self.triangulated_surface)

    @classmethod
    def load(cls, filename: str) -> GeodeTriangulatedSurface3D:
        return GeodeTriangulatedSurface3D(og.load_triangulated_surface3D(filename))

    @classmethod
    def additional_files(cls, filename: str) -> og.AdditionalFiles:
        return og.triangulated_surface_additional_files3D(filename)

    @classmethod
    def is_loadable(cls, filename: str) -> og.Percentage:
        return og.is_triangulated_surface_loadable3D(filename)

    @classmethod
    def input_extensions(cls) -> list[str]:
        return og.TriangulatedSurfaceInputFactory3D.list_creators()

    @classmethod
    def output_extensions(cls) -> list[str]:
        return og.TriangulatedSurfaceOutputFactory3D.list_creators()

    @classmethod
    def object_priority(cls, filename: str) -> int:
        return og.triangulated_surface_object_priority3D(filename)

    def is_saveable(self, filename: str) -> bool:
        return og.is_triangulated_surface_saveable3D(
            self.triangulated_surface, filename
        )

    def save(self, filename: str) -> list[str]:
        return og.save_triangulated_surface3D(self.triangulated_surface, filename)

    def save_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_viewable_triangulated_surface3D(
            self.triangulated_surface, filename_without_extension
        )

    def save_light_viewable(self, filename_without_extension: str) -> str:
        return viewables.save_light_viewable_triangulated_surface3D(
            self.triangulated_surface, filename_without_extension
        )



# Standard library imports
import re
import os

# Third party imports
import flask
from flask.ctx import AppContext
from flask.testing import FlaskClient
import shutil
import uuid
import zipfile
from pathlib import Path

# Local application imports
from opengeodeweb_microservice.database.data import Data
from opengeodeweb_back import utils_functions
from opengeodeweb_back.geode_objects.geode_brep import GeodeBRep

base_dir = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.join(base_dir, "data")


def test_increment_request_counter(app_context: AppContext) -> None:
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0
    utils_functions.increment_request_counter(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1


def test_decrement_request_counter(app_context: AppContext) -> None:
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1
    utils_functions.decrement_request_counter(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0


def test_update_last_request_time(app_context: AppContext) -> None:
    LAST_REQUEST_TIME = flask.current_app.config.get("LAST_REQUEST_TIME")
    utils_functions.update_last_request_time(flask.current_app)
    assert flask.current_app.config.get("LAST_REQUEST_TIME", 0) >= LAST_REQUEST_TIME


def test_before_request(app_context: AppContext) -> None:
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0
    utils_functions.before_request(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1


def test_teardown_request(app_context: AppContext) -> None:
    LAST_REQUEST_TIME = flask.current_app.config.get("LAST_REQUEST_TIME")
    assert flask.current_app.config.get("REQUEST_COUNTER") == 1
    utils_functions.teardown_request(flask.current_app)
    assert flask.current_app.config.get("REQUEST_COUNTER") == 0
    assert flask.current_app.config.get("LAST_REQUEST_TIME", 0) >= LAST_REQUEST_TIME


def test_versions() -> None:
    list_packages = [
        "OpenGeode-core",
        "OpenGeode-IO",
        "OpenGeode-Geosciences",
        "OpenGeode-GeosciencesIO",
    ]
    versions = utils_functions.versions(list_packages)
    assert type(versions) is list
    for version in versions:
        assert type(version) is dict


def test_extension_from_filename() -> None:
    extension = utils_functions.extension_from_filename("test.toto")
    assert type(extension) is str
    assert extension.count(".") == 0


def test_handle_exception(client: FlaskClient) -> None:
    route = "/error"
    response = client.post(route)
    assert response.status_code == 500
    data = response.get_json()
    assert type(data) is dict
    assert type(data["description"]) is str
    assert type(data["name"]) is str
    assert type(data["code"]) is int


def test_create_data_folder_from_id(client: FlaskClient) -> None:
    app = client.application
    with app.app_context():
        test_id = str(uuid.uuid4()).replace("-", "")
        data_path = utils_functions.create_data_folder_from_id(test_id)
        assert isinstance(data_path, str)
        assert os.path.exists(data_path)
        assert data_path.startswith(flask.current_app.config["DATA_FOLDER_PATH"])
        assert test_id in data_path
        shutil.rmtree(data_path, ignore_errors=True)
        assert not os.path.exists(data_path)


def test_save_all_viewables_and_return_info(client: FlaskClient) -> None:
    app = client.application
    with app.app_context():
        expected_db_path = os.path.join(data_dir, "project.db")
        expected_uri = f"sqlite:///{expected_db_path}"

        assert app.config["SQLALCHEMY_DATABASE_URI"] == expected_uri
        assert os.path.exists(expected_db_path)

        geode_object = GeodeBRep.load(os.path.join(data_dir, "test.og_brep"))

        data_entry = Data.create(
            geode_object=geode_object.geode_object_type(),
            viewer_object=geode_object.viewer_type(),
            viewer_elements_type=geode_object.viewer_elements_type(),
        )
        data_path = utils_functions.create_data_folder_from_id(data_entry.id)

        result = utils_functions.save_all_viewables_and_return_info(
            geode_object, data_entry, data_path
        )

        assert isinstance(result, dict)
        native_file = result["native_file"]
        assert isinstance(native_file, str)
        assert native_file == "native.og_brep"
        viewable_file = result["viewable_file"]
        assert isinstance(viewable_file, str)
        assert viewable_file.endswith(".vtm")
        assert isinstance(result["id"], str)
        assert len(result["id"]) == 32
        assert re.match(r"[0-9a-f]{32}", result["id"])
        assert isinstance(result["viewer_type"], str)
        assert isinstance(result["binary_light_viewable"], str)
        assert result["geode_object_type"] == geode_object.geode_object_type()

        db_entry = Data.get(result["id"])
        assert db_entry is not None
        assert db_entry.native_file == result["native_file"]
        assert db_entry.viewable_file == result["viewable_file"]
        assert db_entry.geode_object == geode_object.geode_object_type()

        expected_data_path = os.path.join(app.config["DATA_FOLDER_PATH"], result["id"])
        assert os.path.exists(expected_data_path)


def test_save_all_viewables_commits_to_db(client: FlaskClient) -> None:
    app = client.application
    with app.app_context():
        geode_object = GeodeBRep.load(os.path.join(data_dir, "test.og_brep"))
        data_entry = Data.create(
            geode_object=geode_object.geode_object_type(),
            viewer_object=geode_object.viewer_type(),
            viewer_elements_type=geode_object.viewer_elements_type(),
        )
        data_path = utils_functions.create_data_folder_from_id(data_entry.id)

        result = utils_functions.save_all_viewables_and_return_info(
            geode_object, data_entry, data_path
        )
        data_id = result["id"]
        assert isinstance(data_id, str)
        db_entry_before = Data.get(data_id)
        assert db_entry_before is not None
        assert db_entry_before.native_file == result["native_file"]


def test_generate_native_viewable_and_light_viewable_from_object(
    client: FlaskClient,
) -> None:
    app = client.application
    with app.app_context():
        geode_object = GeodeBRep.load(os.path.join(data_dir, "test.og_brep"))

        result = (
            utils_functions.generate_native_viewable_and_light_viewable_from_object(
                geode_object
            )
        )

        assert isinstance(result, dict)
        assert isinstance(result["native_file"], str)
        assert result["native_file"].startswith("native.")
        assert isinstance(result["viewable_file"], str)
        assert result["viewable_file"].endswith(".vtm")
        assert isinstance(result["id"], str)
        assert re.match(r"[0-9a-f]{32}", result["id"])
        assert isinstance(result["viewer_type"], str)
        assert isinstance(result["binary_light_viewable"], str)
        assert result["binary_light_viewable"].startswith('<?xml version="1.0"?>')

        data = Data.get(result["id"])
        assert data is not None
        assert data.light_viewable_file is not None
        assert data.light_viewable_file.endswith(".vtp")

        data_path = os.path.join(app.config["DATA_FOLDER_PATH"], result["id"])
        assert os.path.exists(os.path.join(data_path, result["native_file"]))
        assert os.path.exists(os.path.join(data_path, result["viewable_file"]))
        assert os.path.exists(os.path.join(data_path, data.light_viewable_file))


def test_generate_native_viewable_and_light_viewable_from_file(
    client: FlaskClient,
) -> None:
    app = client.application
    with app.app_context():
        result = utils_functions.generate_native_viewable_and_light_viewable_from_file(
            GeodeBRep.geode_object_type(), "test.og_brep"
        )

    assert isinstance(result, dict)
    assert isinstance(result["native_file"], str)
    assert result["native_file"] == "native.og_brep"
    assert isinstance(result["viewable_file"], str)
    assert result["viewable_file"].endswith(".vtm")
    assert isinstance(result["id"], str)
    assert re.match(r"[0-9a-f]{32}", result["id"])
    assert isinstance(result["viewer_type"], str)
    assert isinstance(result["binary_light_viewable"], str)


def test_send_file_multiple_returns_zip(client: FlaskClient, tmp_path: Path) -> None:
    app = client.application
    with app.app_context():
        app.config["UPLOAD_FOLDER"] = str(tmp_path)
        file_paths = []
        for i, content in [(1, b"hello 1"), (2, b"hello 2")]:
            file_path = tmp_path / f"tmp_send_file_{i}.txt"
            file_path.write_bytes(content)
            file_paths.append(str(file_path))
        with app.test_request_context():
            response = utils_functions.send_file(
                app.config["UPLOAD_FOLDER"], file_paths, "bundle"
            )
            assert response.status_code == 200
            assert response.mimetype == "application/zip"
            new_file_name = response.headers.get("new-file-name")
            assert new_file_name == "bundle.zip"
            zip_path = os.path.join(app.config["UPLOAD_FOLDER"], new_file_name)
            with zipfile.ZipFile(zip_path, "r") as zip_file:
                zip_entries = zip_file.namelist()
                assert "tmp_send_file_1.txt" in zip_entries
                assert "tmp_send_file_2.txt" in zip_entries
            response.close()


def test_send_file_single_returns_octet_binary(
    client: FlaskClient, tmp_path: Path
) -> None:
    app = client.application
    with app.app_context():
        app.config["UPLOAD_FOLDER"] = str(tmp_path)
        file_path = tmp_path / "tmp_send_file_1.txt"
        file_path.write_bytes(b"hello 1")
        with app.test_request_context():
            response = utils_functions.send_file(
                app.config["UPLOAD_FOLDER"], [str(file_path)], "tmp_send_file_1.txt"
            )
            assert response.status_code == 200
            assert response.mimetype == "application/octet-binary"
            new_file_name = response.headers.get("new-file-name")
            assert new_file_name == "tmp_send_file_1.txt"
            zip_path = os.path.join(app.config["UPLOAD_FOLDER"], new_file_name)
            with open(zip_path, "rb") as f:
                file_bytes = f.read()
            assert file_bytes == b"hello 1"
            response.close()



# Standard library imports
import os
import time

# Third party imports
# Local application imports


class Config(object):
    FLASK_DEBUG = os.environ.get("FLASK_DEBUG", default=False)
    DEFAULT_HOST = "localhost"
    DEFAULT_PORT = "5000"
    CORS_HEADERS = "Content-Type"
    UPLOAD_FOLDER = "./uploads"
    REQUEST_COUNTER = 0
    LAST_REQUEST_TIME = time.time()
    LAST_PING_TIME = time.time()
    DATABASE_FILENAME = "project.db"


class ProdConfig(Config):
    SSL = None
    ORIGINS = ""
    MINUTES_BEFORE_TIMEOUT = "1"
    SECONDS_BETWEEN_SHUTDOWNS = "10"
    DATA_FOLDER_PATH = "/data"
    EXTENSIONS_FOLDER_PATH = os.path.join(DATA_FOLDER_PATH, "extensions")


class DevConfig(Config):
    SSL = None
    ORIGINS = "*"
    MINUTES_BEFORE_TIMEOUT = "1"
    SECONDS_BETWEEN_SHUTDOWNS = "10"
    BASE_DIR = os.path.dirname(os.path.abspath(__file__))
    DATA_FOLDER_PATH = os.path.join(BASE_DIR, "data")
    EXTENSIONS_FOLDER_PATH = os.path.join(DATA_FOLDER_PATH, "extensions")



# Standard library imports
from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Any

# Third party imports
import opengeode as og
from opengeodeweb_microservice.database.data_types import (
    GeodeObjectType,
    ViewerType,
    ViewerElementsType,
)

# Local application imports


class GeodeObject(ABC):
    identifier: og.Identifier

    def __init__(self, identifier: og.Identifier | None = None) -> None:
        self.identifier = identifier if identifier is not None else og.Identifier()

    @classmethod
    @abstractmethod
    def geode_object_type(cls) -> GeodeObjectType: ...

    @classmethod
    @abstractmethod
    def viewer_type(cls) -> ViewerType: ...

    @classmethod
    @abstractmethod
    def viewer_elements_type(cls) -> ViewerElementsType: ...

    @classmethod
    @abstractmethod
    def is_3D(cls) -> bool: ...

    @classmethod
    @abstractmethod
    def is_viewable(cls) -> bool: ...

    @abstractmethod
    def builder(self) -> Any: ...

    @classmethod
    @abstractmethod
    def is_loadable(cls, filename: str) -> og.Percentage: ...

    @classmethod
    @abstractmethod
    def load(cls, filename: str) -> GeodeObject: ...

    @classmethod
    @abstractmethod
    def additional_files(cls, filename: str) -> og.AdditionalFiles: ...

    @abstractmethod
    def native_extension(cls) -> str: ...

    @classmethod
    @abstractmethod
    def input_extensions(cls) -> list[str]: ...

    @classmethod
    @abstractmethod
    def object_priority(cls, filename: str) -> int: ...

    @classmethod
    @abstractmethod
    def output_extensions(cls) -> list[str]: ...

    @abstractmethod
    def is_saveable(self, filename: str) -> bool: ...

    @abstractmethod
    def save(self, filename: str) -> list[str]: ...

    @abstractmethod
    def save_viewable(self, filename_without_extension: str) -> str: ...

    @abstractmethod
    def save_light_viewable(self, filename_without_extension: str) -> str: ...

    @abstractmethod
    def inspect(self) -> Any: ...
